--Maximum view distance based on player counts
Distances = {11, 10, 9, 9, 8, 8, 7}
NumDistances = 7

--Helper variables (DO NOT CHANGE!!!)
MaxView = Distances[1]
GlobalView = Distances[1]

ChunkQueue = 0 -- Keep 0
MaxQueue = 1 -- 1-inf, higher number is worse performance
ViewCheckTimer = 0 -- Keep 0
ViewCheckTime = 10 -- 1-inf, system speed determines best number, 10 is good


--View Distance Penalties For Movement (CHANGE BASED ON SERVER SPEED)
FlyRunningPenalty = 9
FlyingPenalty = 7
RunJumpingPenalty = 5
RunningPenalty = 4
WalkingPenalty= 2
SlowWalkPenalty= 1


--Speed Thresholds For Movement (DO NOT CHANGE!!!)
FlyRunningSpeed = 1
FlyingSpeed = 0.5
RunJumpingSpeed = 0.3
RunningSpeed = 0.25
WalkingSpeed = 0.2
SlowWalkSpeed = 0.0


function Initialize(Plugin)
    Plugin:SetName("PiChunks")
    Plugin:SetVersion(1)

    cPluginManager.AddHook(cPluginManager.HOOK_PLAYER_SPAWNED, OnPlayerSpawned)
    cPluginManager.AddHook(cPluginManager.HOOK_ENTITY_TELEPORT, OnPlayerTeleport)    
    cPluginManager.AddHook(cPluginManager.HOOK_ENTITY_CHANGING_WORLD, OnPlayerChangeWorlds)    
    cPluginManager.AddHook(cPluginManager.HOOK_PLAYER_MOVING, OnPlayerMove)
    cPluginManager.AddHook(cPluginManager.HOOK_CHUNK_AVAILABLE, OnChunkUpdate)

    LOG("Initialized "..Plugin:GetName().." v."..Plugin:GetVersion())
    return true
end 

function OnPlayerSpawned(Player)
    SetPlayerView(Player, MaxView - FlyRunningPenalty)
    return false
end

function OnPlayerTeleport(Entity)
    if Entity:IsPlayer() then
        SetPlayerView(Entity, MaxView - FlyRunningPenalty)
    end
    return false
end

function OnPlayerChangeWorlds(Entity, World)
    if Entity:IsPlayer() then
        SetPlayerView(Entity, MaxView - FlyRunningPenalty)
    end
    return false
end

function OnPlayerMove(Player, OldPos, NewPos)
    Speed = math.sqrt(math.pow(NewPos.x - OldPos.x , 2) + math.pow(NewPos.z - OldPos.z, 2))
    if Speed > FlyRunningSpeed then
        SetPlayerView(Player, MaxView - FlyRunningPenalty)
    elseif Speed > FlyingSpeed then
        SetPlayerView(Player, MaxView - FlyingPenalty)
    elseif Speed > RunJumpingSpeed then
        SetPlayerView(Player, MaxView - RunJumpingPenalty)
    elseif Speed > RunningSpeed then
        SetPlayerView(Player, MaxView - RunningPenalty)
    elseif Speed > WalkingSpeed then
        SetPlayerView(Player, MaxView - WalkingPenalty)
    elseif Speed > SlowWalkSpeed then
        SetPlayerView(Player, MaxView - SlowWalkPenalty)
    else
        SetPlayerView(Player, MaxView)
    end
    return false
end

function OnChunkUpdate(World, ChunkX, ChunkZ, ChunkDesc)
    World:SetMaxViewDistance(GlobalView)

    ChunkQueue = World:GetStorageLoadQueueLength() + World:GetGeneratorQueueLength()

    NumPlayers = cRoot:Get():GetServer():GetNumPlayers()

    if NumPlayers == 0 then
        World:SetMaxViewDistance(1)
    end
    if NumPlayers > 0 and NumPlayers <= NumDistances then
        MaxView = Distances[NumPlayers]
    end

    return false
end

function SetPlayerView(Player, Distance)
    Client = Player:GetClientHandle()
    if Client:GetViewDistance() > Distance then
       Client:SetViewDistance(Distance)
    elseif Client:GetViewDistance() < Distance and ChunkQueue < MaxQueue then
       ViewCheckTimer = ViewCheckTimer + 1
       if ViewCheckTimer > ViewCheckTime then
           ViewCheckTimer = 0
           Client:SetViewDistance(Client:GetViewDistance() + 1)
       end
    end
end 
